package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.RollCallEmployeeCriteria;
import com.bcxin.tenant.open.domains.criterias.RollCallEmployeeGroupCriteria;
import com.bcxin.tenant.open.domains.dtos.RollCallEmployeeGroupSearchDTO;
import com.bcxin.tenant.open.domains.entities.RollCallEmployeeEntity;
import com.bcxin.tenant.open.domains.mappers.RollCallEmployeeMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.enums.RollCallStatus;
import com.bcxin.tenant.open.infrastructures.enums.RollCallType;
import com.bcxin.tenant.open.infrastructures.exceptions.NoAllowedTenantException;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Repository
public class RollCallEmployeeRepositoryImpl implements RollCallEmployeeRepository {
    private final RollCallEmployeeMapper rollCallEmployeeMapper;

    public RollCallEmployeeRepositoryImpl(RollCallEmployeeMapper rollCallEmployeeMapper) {
        this.rollCallEmployeeMapper = rollCallEmployeeMapper;
    }

    @Override
    public RollCallEmployeeEntity getById(Object id) {
        return this.rollCallEmployeeMapper.selectById((Long) id);
    }

    @Override
    public void insert(RollCallEmployeeEntity data) {
        this.rollCallEmployeeMapper.insert(data);
    }

    @Override
    public void update(RollCallEmployeeEntity data) {
        this.rollCallEmployeeMapper.updateById(data);
    }

    @Override
    public void batchInsert(Collection<RollCallEmployeeEntity> data) {
        for (RollCallEmployeeEntity ee : data) {
            this.rollCallEmployeeMapper.insert(ee);
        }
    }

    @Override
    public Collection<RollCallEmployeeEntity> findByRollCallIdAndEmployeeIds(Long rollCallId, Collection<String> employeeIds) {
        if (CollectionUtils.isEmpty(employeeIds) || rollCallId == null) {
            return Collections.EMPTY_LIST;
        }

        LambdaQueryWrapper<RollCallEmployeeEntity> lambdaQueryWrapper
                = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallEmployeeEntity::getRollCallId, rollCallId);
        lambdaQueryWrapper = lambdaQueryWrapper.in(RollCallEmployeeEntity::getCalledEmployeeId, employeeIds);

        return this.rollCallEmployeeMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public EntityCollection<RollCallEmployeeGroupSearchDTO> search(RollCallEmployeeGroupCriteria criteria) {
        Collection<RollCallEmployeeGroupSearchDTO> data = this.rollCallEmployeeMapper.searchByGroup(criteria);
        long totalCount = this.rollCallEmployeeMapper.getCountOfSearchByGroup(criteria);

        return EntityCollection.create(data, criteria.getPageSize(), totalCount);
    }

    @Override
    public Collection<RollCallEmployeeEntity> search(RollCallEmployeeCriteria criteria) {
        LambdaQueryWrapper<RollCallEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.hasLength(criteria.getCalledEmployeeId())) {
            lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallEmployeeEntity::getCalledEmployeeId, criteria.getCalledEmployeeId());
        }

        if (StringUtils.hasLength(criteria.getRollCallId())) {
            lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallEmployeeEntity::getRollCallId, criteria.getRollCallId());
        }

        if (criteria.getBeginDate() != null) {
            lambdaQueryWrapper = lambdaQueryWrapper.ge(RollCallEmployeeEntity::getCreatedTime, criteria.getBeginDate());
        }

        if (StringUtils.hasLength(criteria.getActionOrganizationId())) {
            lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallEmployeeEntity::getActionOrganizationId, criteria.getActionOrganizationId());
        }

        if (StringUtils.hasLength(criteria.getSecurityStationId())) {
            String stationId = criteria.getSecurityStationId().replace("'", "");
            String stationIdWhere = " security_station_json->'$.id'='" + stationId + "'";
            lambdaQueryWrapper = lambdaQueryWrapper.apply(stationIdWhere);
        }

        if (criteria.getEndDate() != null) {
            Date endDate = criteria.getEndDate();
            Calendar endDateCalender = Calendar.getInstance();
            endDateCalender.setTime(endDate);
            endDateCalender.add(Calendar.DAY_OF_MONTH, 1);
            endDate = endDateCalender.getTime();

            lambdaQueryWrapper = lambdaQueryWrapper.lt(RollCallEmployeeEntity::getCreatedTime, endDate);
        }

        if (!CollectionUtils.isEmpty(criteria.getCallStatuses())) {
            int callStatusValue = RollCallStatus.calculate(criteria.getCallStatuses());
            lambdaQueryWrapper = lambdaQueryWrapper.apply("`status` & " + callStatusValue + ">0");
        }

        if (!CollectionUtils.isEmpty(criteria.getCallTypes())) {
            int callTypeValue = RollCallType.calculate(criteria.getCallTypes());
            lambdaQueryWrapper = lambdaQueryWrapper.apply("`call_type` & " + callTypeValue + ">0");
        }

        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s,%s", criteria.getSkip(), criteria.getPageSize()));

        Collection<RollCallEmployeeEntity> data =
                this.rollCallEmployeeMapper.selectList(lambdaQueryWrapper);

        return data;
    }

    @Override
    public Collection<RollCallEmployeeEntity> getTopNCanBeExpiredRecords(int pageSize,RollCallStatus callStatus) {
        if(callStatus!=RollCallStatus.RollCallInit && callStatus!=RollCallStatus.DispatchInit) {
            throw new NoAllowedTenantException();
        }

        LambdaQueryWrapper<RollCallEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        Calendar currentCalendar = Calendar.getInstance();
        int seconds = 3 * 60;
        if (callStatus == RollCallStatus.RollCallInit) {
            seconds = 3 * 60;
        } else {
            seconds = 66;
        }

        currentCalendar.add(Calendar.SECOND, 0 - seconds);
        Date shouldBeExpiredTime = currentCalendar.getTime();

        lambdaQueryWrapper = lambdaQueryWrapper.le(RollCallEmployeeEntity::getLastActionTime, shouldBeExpiredTime);

        Collection<RollCallStatus> selectedRollCallStatuses = Stream.of(callStatus).collect(Collectors.toList());
        int calculatedRollCallStatus = RollCallStatus.calculate(selectedRollCallStatuses);
        lambdaQueryWrapper = lambdaQueryWrapper.apply("status & " + calculatedRollCallStatus + " > 0");
        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s", pageSize));

        return this.rollCallEmployeeMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public Collection<RollCallEmployeeEntity> findByIds(Collection<String> ids) {
        return this.rollCallEmployeeMapper.selectBatchIds(ids);
    }

    @Override
    public Collection<RollCallEmployeeEntity> findByRollCallIdsAndEmployeeIds(Collection<Long> rollCallIds, Collection<String> employeeIds) {
        if (CollectionUtils.isEmpty(employeeIds) || CollectionUtils.isEmpty(rollCallIds)) {
            return Collections.EMPTY_LIST;
        }

        LambdaQueryWrapper<RollCallEmployeeEntity> lambdaQueryWrapper
                = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.in(RollCallEmployeeEntity::getRollCallId, rollCallIds);
        lambdaQueryWrapper = lambdaQueryWrapper.in(RollCallEmployeeEntity::getCalledEmployeeId, employeeIds);

        return this.rollCallEmployeeMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public void batchUpdateStatusAndCId(Collection<RollCallEmployeeEntity> data) {
        this.rollCallEmployeeMapper.batchUpdateStatusAndCId(data);
    }
}
